/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBEGUIPLUGIN_POP_HYBRID_PARALLEL_EFFICIENCY_TEST_H
#define CUBEGUIPLUGIN_POP_HYBRID_PARALLEL_EFFICIENCY_TEST_H

#include <string>
#include "PerformanceTest.h"
#include "POPHybridProcessEfficiencyTest.h"
#include "POPHybridThreadEfficiencyTest.h"

namespace hybanalysis
{
class POPHybridParallelEfficiencyTest : public popcalculation::PerformanceTest
{
private:
    cube::Metric* pop_avg_comp;
    cube::Metric* max_runtime;

    cube::list_of_metrics max_runtime_metrics;


    POPHybridProcessEfficiencyTest* proc_eff;
    POPHybridThreadEfficiencyTest*  thread_eff;

    void
    calculate();

    void
    calculate( const cube::list_of_cnodes& );

protected:

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false  );


    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "Parallel Efficiency (PE) reveals the inefficiency in processes and threads utilization. \n"
            "These are measured with Process Efficiency and Thread Efficiency, \n"
            "and PE can be computed directly or as a product of these two sub-metrics:\n"
            "\tPE = avg(comp) / max(runtime) \n"
            "\t   = Process Efficiency x Thread Efficiency";
        std::string not_active_text =
            "Parallel Efficiency metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;


public:
    POPHybridParallelEfficiencyTest(  cube::CubeProxy*,
                                      POPHybridProcessEfficiencyTest* proc_eff,
                                      POPHybridThreadEfficiencyTest*  thread_eff );

    virtual
    ~POPHybridParallelEfficiencyTest()
    {
    };
// ------ overview tests ---------

    bool
    isActive() const;

    bool
    isIssue() const;


    virtual
    std::list<PerformanceTest*>
    getPrereqs()
    {
        std::list<PerformanceTest*> prereqs;
        if ( proc_eff != nullptr && thread_eff != nullptr )
        {
            prereqs.push_back( proc_eff );
            prereqs.push_back( thread_eff );
        }
        return prereqs;
    }
};
};
#endif // ADVISER_RATING_WIDGET_H
