/****************************************************************************
**  CUBE        http://www.scalasca.org/                                   **
*****************************************************************************
**  Copyright (c) 2015-2023                                                **
**  Forschungszentrum Juelich GmbH, Juelich Supercomputing Centre          **
**                                                                         **
**  This software may be modified and distributed under the terms of       **
**  a BSD-style license.  See the COPYING file in the package base         **
**  directory for details.                                                 **
****************************************************************************/


#ifndef CUBELIB_PARALLEL_EFFICIENCY_TEST_H
#define CUBELIB_PARALLEL_EFFICIENCY_TEST_H

#include <string>
#include "PerformanceTest.h"
#include "POPImbalanceTest.h"
#include "POPCommunicationEfficiencyTest.h"

namespace mpianalysis
{
class POPParallelEfficiencyTest : public popcalculation::PerformanceTest
{
private:
    POPImbalanceTest*               pop_lb;
    POPCommunicationEfficiencyTest* pop_commeff;


    void
    calculate();


protected:



    virtual
    const std::string&
    getCommentText() const;

    inline
    virtual
    std::string
    getHelpUrl() const
    {
        std::string active_text =
            "Parallel Efficiency (PE) reveals the inefficiency in splitting computation over processes and then \n"
            "communicating data between processes. As with GE, PE is a compound metric whose components reflects \n"
            "two important factors in achieving good parallel performance in code:\n\n"

            "\tEnsuring even distribution of computational work across processes\n"
            "\tMinimizing time communicating data between processes\n\n"

            "These are measured with Load Balance Efficiency and Communication Efficiency, \n"
            "and PE is defined as the product of these two sub-metrics:\n"
            "\tPE = Load Balance x Communication Efficiency ";

        std::string not_active_text =
            "Parallel Efficiency metric is a basic POP metric and is available for every Score-P/Scalasca measurement. \n"
            "If Cube Report was produced by another tool than Score-P/Scalasca, it might have missing metric Time.\n"
            "In this case POP analysis is not possible.";

        return active_text;
    }

    virtual
    void
    adjustForTest( cube::CubeProxy* cube ) const;


public:
    POPParallelEfficiencyTest( POPImbalanceTest*               pop_lb,
                               POPCommunicationEfficiencyTest* pop_commeff );

    virtual
    ~POPParallelEfficiencyTest()
    {
    };

    void
    applyCnode( const cube::list_of_cnodes& cnodes,
                const bool                  direct_calculation = false   );

    bool
    isActive() const;

    bool
    isIssue() const;


    virtual
    std::list<PerformanceTest*>
    getPrereqs()
    {
        std::list<PerformanceTest*> prereqs;
        if ( pop_lb != nullptr && pop_commeff != nullptr )
        {
            prereqs.push_back( pop_lb );
            prereqs.push_back( pop_commeff );
        }
        return prereqs;
    }
};
};
#endif // ADVISER_RATING_WIDGET_H
